/*
 * whereami.c: displays information about the machine you are working on.
 *
 * Copyright (C) 2010 Prasad J Pandit
 *
 * `whereami' is a free software; you can redistribute it and/or modify it
 * under the terms of GNU General Public License as published by the Free
 * Software Foundation; either version 3 of the license, or(at your option)
 * any later version.
 *
 * `whereami' is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with `wmi'; if not see: <http://www.gnu.org/licenses>.
 *
 */

#include <err.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <unistd.h>

#include <net/if.h>
#include <inttypes.h>
#include <sys/ioctl.h>

#define CWDLEN       4096
#define VERSION     "1.0"

struct option lopt[] = \
{
    { "dir", no_argument, NULL, 'd' },
    { "help", no_argument, NULL, 'h' },
    { "hostip", no_argument, NULL, 'i' },
    { "hostname", no_argument, NULL, 'n' },
    { "ttyname", no_argument, NULL, 't' },
    { "version", no_argument, NULL, 'v' },

    { NULL, 0, NULL, 0 }
};

extern int opterr;
static char *prog = NULL;

char *getname (void);                   /* get host name */
uint32_t getip (void);                  /* get host ip address */

enum flag { P_DIR = 1, P_NAME = 2, P_IP = 4, P_TTY = 8, P_ALL = 15 };


void
usage (void)
{
    printf ("Usage: %s [OPTIONs]\n", prog);
}


void
printh (void)
{
    usage ();

    printf ("\nOptions\n");
    printf (" %-15s %s\n", "  -d --dir", "display working directory");
    printf (" %-15s %s\n", "  -h --help", "shows this help");
    printf (" %-15s %s\n", "  -i --hostip", "display host ip address");
    printf (" %-15s %s\n", "  -n --hostname", "display host name");
    printf (" %-15s %s\n", "  -t --ttyname", "display terminal name");
    printf (" %-15s %s\n", "  -v --version", "display version information");
    printf ("\nReport bugs to <pj.pandit@yahoo.co.in>\n");
}


int
main (int argc, char *argv[])
{
    uint32_t ip = 0;
    char *s = NULL, optstr[] = "dhintv";
    int n = 0, indx = 0, pflag = P_ALL, oflag = 0;

    prog = strdup (argv[0]);

    opterr = 0;
    while ((n = getopt_long (argc, argv, optstr, lopt, &indx)) != -1)
    {
        switch (n)
        {
        case 'd':
            oflag |= P_DIR;
            break;

        case 'h':
            printh ();
            return 0;

        case 'i':
            oflag |= P_IP;
            break;

        case 'n':
            oflag |= P_NAME;
            break;

        case 't':
            oflag |= P_TTY;
            break;

        case 'v':
            printf ("%s version %s\n", prog, VERSION);
            return 0;

        default:
            errx (1, "invalid option: `-%c', see --help", optopt);
        }
    }
    pflag = (oflag != 0) ? oflag : P_ALL;

    (pflag & P_TTY) ? printf ("%s ", ttyname (STDIN_FILENO)) : 0;

    if (pflag & P_DIR)
    {
        if (!(s = calloc (CWDLEN, sizeof (char))))
            err (1, "insufficient memory");

        printf ("%s ", getcwd (s, CWDLEN));
        free (s);
    }

    if (pflag & P_NAME)
    {
        if (!(s = getname ()))
            err (1, "could not read host name");

        printf ("%s ", s);
        free (s);
    }

    if (pflag & P_IP)
    {
        ip = getip ();

        s = &ip;
        printf ("%hhu.", *s++);
        printf ("%hhu.", *s++);
        printf ("%hhu.", *s++);
        printf ("%hhu", *s);
    }
    printf ("\n");

    free (prog);

    return 0;
}


char *
getname (void)
{
    const int SIZE = 255;
    char *buf = calloc (SIZE, sizeof (char));

    if(!gethostname (buf, SIZE))
        return buf;

    free (buf);
    return NULL;
}


uint32_t
getip (void)
{
    struct ifconf iff;
    struct ifreq ifc[10];

    char *c = NULL;
    uint32_t ip = 0;
    int i = 0, len = 0, s = 0;

    if ((s = socket (AF_INET, SOCK_STREAM, 0)) < 0)
        err (1, "socket failed");

    iff.ifc_req = ifc;
    iff.ifc_len = sizeof (ifc);
    if (ioctl (s, SIOCGIFCONF, &iff) < 0)
        err (1, "ioctl failed - 1");

    len = iff.ifc_len / sizeof (struct ifreq);
    for (i = 0; i < len; i++)
    {
        ip = 0;
        c = ifc[i].ifr_addr.sa_data;
        memcpy (&ip, c + 2, 4);

        ifc[i].ifr_flags = IFF_LOOPBACK;
        if (ioctl (s, SIOCGIFFLAGS, &ifc[i]) < 0)
            err (1, "ioctl failed - 2");

        if (!(ifc[i].ifr_flags & IFF_LOOPBACK))
            break;
    }
    close (s);

    return ip;
}
